@@ -19,15 +19,31 @@
 import gobject
 import locale
 import logging
+import os
 import re
+import string
 import xapian
-
-from softwarecenter import Application
+from softwarecenter.db.application import Application
 
 from softwarecenter.utils import *
 from softwarecenter.enums import *
 from gettext import gettext as _
 
+def parse_axi_values_file(filename="/var/lib/apt-xapian-index/values"):
+    """ parse the apt-xapian-index "values" file and provide the 
+    information in the self._axi_values dict
+    """
+    axi_values = {}
+    if not os.path.exists(filename):
+        return
+    for raw_line in open(filename):
+        line = string.split(raw_line, "#", 1)[0]
+        if line.strip() == "":
+            continue
+        (key, value) = line.split()
+        axi_values[key] = int(value)
+    return axi_values
+
 class StoreDatabase(gobject.GObject):
     """thin abstraction for the xapian database with convenient functions"""
 
@@ -51,8 +67,13 @@ class StoreDatabase(gobject.GObject):
         gobject.GObject.__init__(self)
         self._db_pathname = pathname
         self._aptcache = cache
+        self._additional_databases = []
 
-    def open(self, pathname=None):
+        # the xapian values as read from /var/lib/apt-xapian-index/values
+        self._axi_values = {}
+        self._logger = logging.getLogger("softwarecenter.db")
+
+    def open(self, pathname=None, use_axi=True, use_agent=True):
         " open the database "
         if pathname:
             self._db_pathname = pathname
@@ -60,11 +81,23 @@ class StoreDatabase(gobject.GObject):
         # add the apt-xapian-database for here (we don't do this
         # for now as we do not have a good way to integrate non-apps
         # with the UI)
-        try:
-            axi = xapian.Database("/var/lib/apt-xapian-index/index")
-            self.xapiandb.add_database(axi)
-        except:
-            logging.exception("failed to add apt-xapian-index")
+        if use_axi:
+            try:
+                axi = xapian.Database("/var/lib/apt-xapian-index/index")
+                self.xapiandb.add_database(axi)
+                self._axi_values = parse_axi_values_file()
+            except:
+                self._logger.exception("failed to add apt-xapian-index")
+        if use_agent:
+            try:
+                sca = xapian.Database(XAPIAN_BASE_PATH_SOFTWARE_CENTER_AGENT)
+                self.xapiandb.add_database(sca)
+            except Exception as e:
+                logging.warn("failed to add sca db %s" % e)
+        # additional dbs
+        for db in self._additional_databases:
+            self.xapiandb.add_database(db)
+        # parser etc
         self.xapian_parser = xapian.QueryParser()
         self.xapian_parser.set_database(self.xapiandb)
         self.xapian_parser.add_boolean_prefix("pkg", "XP")
@@ -74,6 +107,13 @@ class StoreDatabase(gobject.GObject):
         self.xapian_parser.set_default_op(xapian.Query.OP_AND)
         self.emit("open", self._db_pathname)
 
+    def add_database(self, database):
+        self._additional_databases.append(database)
+        self.xapiandb.add_database(database)
+
+    def del_database(self, database):
+        self._additional_databases.remove(database)
+
     def reopen(self):
         " reopen the database "
         self.open()
@@ -90,16 +130,24 @@ class StoreDatabase(gobject.GObject):
         https://wiki.ubuntu.com/SoftwareCenter?action=show&redirect=SoftwareStore#Searching%20for%20multiple%20package%20names
         """
         # expand "," to APpkgname AND
+        # (ignore trailing comma)
+        search_term = search_term.rstrip(",")
         if "," in search_term:
-            query = xapian.Query()
+            queries = []
+            added = set()
             for pkgname in search_term.split(","):
-                # not a pkgname
+                pkgname = pkgname.lower()
+                # double comma, ignore term
+                if not pkgname:
+                    continue
+                # not a pkgname, return
                 if not re.match("[0-9a-z\.\-]+", pkgname):
                     return None
-                if pkgname:
-                    query = xapian.Query(xapian.Query.OP_OR, query, 
-                                         xapian.Query("XP"+pkgname))
-            return query
+                # only add if not there already
+                if pkgname not in added:
+                    added.add(pkgname)
+                    queries.append(xapian.Query("AP"+pkgname))
+            return queries
         return None
 
     def get_query_list_from_search_entry(self, search_term, category_query=None):
@@ -126,17 +174,17 @@ class StoreDatabase(gobject.GObject):
         for item in self.SEARCH_GREYLIST_STR.split(";"):
             (search_term, n) = re.subn('\\b%s\\b' % item, '', search_term)
             if n: 
-                logging.debug("greylist changed search term: '%s'" % search_term)
+                self._logger.debug("greylist changed search term: '%s'" % search_term)
         # restore query if it was just greylist words
         if search_term == '':
-            logging.debug("grey-list replaced all terms, restoring")
+            self._logger.debug("grey-list replaced all terms, restoring")
             search_term = orig_search_term
         
         # check if we need to do comma expansion instead of a regular
         # query
-        query = self._comma_expansion(search_term)
-        if query:
-            return _add_category_to_query(query)
+        queries = self._comma_expansion(search_term)
+        if queries:
+            return map(_add_category_to_query, queries)
 
         # get a pkg query
         pkg_query = xapian.Query()
@@ -150,6 +198,12 @@ class StoreDatabase(gobject.GObject):
         fuzzy_query = self.xapian_parser.parse_query(search_term, 
                                                xapian.QueryParser.FLAG_PARTIAL|
                                                xapian.QueryParser.FLAG_BOOLEAN)
+        # if the query size goes out of hand, omit the FLAG_PARTIAL
+        # (LP: #634449)
+        if fuzzy_query.get_length() > 1000:
+            fuzzy_query = self.xapian_parser.parse_query(search_term, 
+                                            xapian.QueryParser.FLAG_BOOLEAN)
+        # now add categories
         fuzzy_query = _add_category_to_query(fuzzy_query)
         return [pkg_query,fuzzy_query]
 
@@ -161,7 +215,7 @@ class StoreDatabase(gobject.GObject):
         # from the apt cache
         if not summary and self._aptcache.ready: 
             pkgname = self.get_pkgname(doc)
-            if (self._aptcache.has_key(pkgname) and 
+            if (pkgname in self._aptcache and 
                 self._aptcache[pkgname].candidate):
                 return  self._aptcache[pkgname].candidate.summary
             elif channel:
@@ -175,11 +229,40 @@ class StoreDatabase(gobject.GObject):
         """ Return a packagename from a xapian document """
         pkgname = doc.get_value(XAPIAN_VALUE_PKGNAME)
         # if there is no value it means we use the apt-xapian-index 
-        # that store the pkgname in the data field directly
+        # that stores the pkgname in the data field directly
         if not pkgname:
             pkgname = doc.get_data()
         return pkgname
 
+    def get_appname(self, doc):
+        """ Return a appname from a xapian document, or None if
+            a value for appname cannot be found in the document
+         """
+        return doc.get_value(XAPIAN_VALUE_APPNAME)
+
+    def get_iconname(self, doc):
+        """ Return the iconname from the xapian document """
+        iconname = doc.get_value(XAPIAN_VALUE_ICON)
+        return iconname
+
+    def pkg_in_category(self, pkgname, cat_query):
+        """ Return True if the given pkg is in the given category """
+        pkg_query1 = xapian.Query("AP"+pkgname)
+        pkg_query2 = xapian.Query("XP"+pkgname)
+        pkg_query = xapian.Query(xapian.Query.OP_OR, pkg_query1, pkg_query2)
+        pkg_and_cat_query = xapian.Query(xapian.Query.OP_AND, pkg_query, cat_query)
+        enquire = xapian.Enquire(self.xapiandb)
+        enquire.set_query(pkg_and_cat_query)
+        matches = enquire.get_mset(0, len(self))
+        if matches:
+            return True
+        return False
+
+        
+    def get_icon_needs_download(self, doc):
+        """ Return a value if the icon needs to be downloaded """
+        return doc.get_value(XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD)
+
     def get_popcon(self, doc):
         """ Return a popcon value from a xapian document """
         popcon_raw = doc.get_value(XAPIAN_VALUE_POPCON)
@@ -194,12 +277,17 @@ class StoreDatabase(gobject.GObject):
         
         If no document is found, raise a IndexError
         """
-        #logging.debug("get_xapian_document app='%s' pkg='%s'" % (appname,pkgname))
+        #self._logger.debug("get_xapian_document app='%s' pkg='%s'" % (appname,pkgname))
         # first search for appname in the app-install-data namespace
         for m in self.xapiandb.postlist("AA"+appname):
             doc = self.xapiandb.get_document(m.docid)
             if doc.get_value(XAPIAN_VALUE_PKGNAME) == pkgname:
                 return doc
+        # then search for pkgname in the app-install-data namespace
+        for m in self.xapiandb.postlist("AP"+pkgname):
+            doc = self.xapiandb.get_document(m.docid)
+            if doc.get_value(XAPIAN_VALUE_PKGNAME) == pkgname:
+                return doc
         # then look for matching packages from a-x-i
         for m in self.xapiandb.postlist("XP"+pkgname):
             doc = self.xapiandb.get_document(m.docid)
@@ -220,6 +308,11 @@ class StoreDatabase(gobject.GObject):
         """return the doc count of the database"""
         return self.xapiandb.get_doccount()
 
+    def __iter__(self):
+        """ support iterating over the documents """
+        for it in self.xapiandb.postlist(""):
+            doc = self.xapiandb.get_document(it.docid)
+            yield doc
 
 if __name__ == "__main__":
     import apt
@@ -237,7 +330,7 @@ if __name__ == "__main__":
     enquire.set_query(query)
     matches = enquire.get_mset(0, len(db))
     for m in matches:
-        doc = m[xapian.MSET_DOCUMENT]
+        doc = m.document
         print doc.get_data()
 
     # test origin
