@@ -16,11 +16,10 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
-import logging
+import apt
 import gtk
 import gobject
-import apt
+import logging
 import os
 import xapian
 
@@ -30,7 +29,7 @@ class InstalledFilter(object):
     def __init__(self, cache):
         self.cache = cache
     def filter(self, pkgname):
-        if self.cache.has_key(pkgname) and self.cache[pkgname].isInstalled:
+        if pkgname in self.cache and self.cache[pkgname].is_installed:
             return True
         return False
 
