@@ -40,15 +40,18 @@ class InstalledPane(SoftwarePane):
     (PAGE_APPLIST,
      PAGE_APP_DETAILS) = range(2)
 
-    def __init__(self, cache, db, distro, icons, datadir):
+    def __init__(self, cache, history, db, distro, icons, datadir):
         # parent
-        SoftwarePane.__init__(self, cache, db, distro, icons, datadir, show_ratings=False)
+        SoftwarePane.__init__(self, cache, history, db, distro, icons, datadir, show_ratings=False)
         # state
         self.apps_filter = AppViewFilter(db, cache)
         self.apps_filter.set_installed_only(True)
         self.current_appview_selection = None
+        self.loaded = False
         # UI
         self._build_ui()
+        self.connect("app-list-changed", self._on_app_list_changed)
+        
     def _build_ui(self):
         self.navigation_bar.set_size_request(26, -1)
         self.notebook.append_page(self.scroll_app_list, gtk.Label("installed"))
@@ -56,7 +59,6 @@ class InstalledPane(SoftwarePane):
         self.notebook.append_page(self.scroll_details, gtk.Label("details"))
         # initial refresh
         self.search_terms = ""
-        self.refresh_apps()
 
     def _show_installed_overview(self):
         " helper that goes back to the overview page "
@@ -74,27 +76,36 @@ class InstalledPane(SoftwarePane):
         """refresh the applist after search changes and update the 
            navigation bar
         """
+        self.loaded = True
+        self.show_appview_spinner()
         if self.search_terms:
             query = self.db.get_query_list_from_search_entry(self.search_terms)
             self.navigation_bar.add_with_id(_("Search Results"),
                                             self.on_navigation_search, 
                                             "search")
         else:
+            # None will default to match all documents (see AppStore code)
             query = None
         self.navigation_bar.add_with_id(_("Installed Software"), 
                                         self.on_navigation_list,
                                         "list",
                                         animate=False)
-        # deactivate the old model, otherwise we have a memleak and
-        # a cpu leak
-        self.app_view.clear_model()
+        # *ugh* deactivate the old model because otherwise it keeps
+        # getting progress_changed events and eats CPU time until it's
+        # garbage collected
+        old_model = self.app_view.get_model()
+        if old_model is not None:
+            old_model.active = False
         # get a new store and attach it to the view
         new_model = AppStore(self.cache,
                              self.db, 
                              self.icons, 
-                             query, 
+                             query,
+                             nonapps_visible = self.nonapps_visible,
                              filter=self.apps_filter)
         self.app_view.set_model(new_model)
+        self.app_view.get_model().active = True
+        self.hide_appview_spinner()
         self.emit("app-list-changed", len(new_model))
         return False
     def on_search_terms_changed(self, searchentry, terms):
@@ -107,7 +118,7 @@ class InstalledPane(SoftwarePane):
         self.notebook.set_current_page(self.PAGE_APPLIST)
     def on_db_reopen(self, db):
         self.refresh_apps()
-        self._show_installed_overview()
+        self.app_details.refresh_app()
         
     def on_navigation_search(self, pathbar, part):
         """ callback when the navigation button with id 'search' is clicked"""
@@ -115,6 +126,8 @@ class InstalledPane(SoftwarePane):
         
     def on_navigation_list(self, pathbar, part):
         """callback when the navigation button with id 'list' is clicked"""
+        if not self.loaded:
+            self.refresh_apps()
         if not pathbar.get_active():
             return
         self._clear_search()
@@ -130,11 +143,18 @@ class InstalledPane(SoftwarePane):
             return
         self.notebook.set_current_page(self.PAGE_APP_DETAILS)
         self.searchentry.hide()
+        self.action_bar.clear()
         
     def on_application_selected(self, appview, app):
         """callback when an app is selected"""
         logging.debug("on_application_selected: '%s'" % app)
         self.current_appview_selection = app
+        
+    def _on_app_list_changed(self, pane, length):
+        """internal helper that keeps the the action bar up-to-date by
+           keeping track of the app-list-changed signals
+        """
+        self.update_show_hide_nonapps()
 
     def display_search(self):
         self.navigation_bar.remove_id("details")
@@ -155,13 +175,13 @@ class InstalledPane(SoftwarePane):
             return ""
         length = len(model)
         if len(self.searchentry.get_text()) > 0:
-            return gettext.ngettext("%s matching item",
-                                    "%s matching items",
-                                    length) % length
+            return gettext.ngettext("%(amount)s matching item",
+                                    "%(amount)s matching items",
+                                    length) % { 'amount' : length, }
         else:
-            return gettext.ngettext("%s item installed",
-                                    "%s items installed",
-                                    length) % length
+            return gettext.ngettext("%(amount)s item installed",
+                                    "%(amount)s items installed",
+                                    length) % { 'amount' : length, }
                                     
     def get_current_app(self):
         """return the current active application object applicable
@@ -171,11 +191,29 @@ class InstalledPane(SoftwarePane):
     def is_category_view_showing(self):
         # there is no category view in the installed pane
         return False
+        
+    def is_applist_view_showing(self):
+        """Return True if we are in the applist view """
+        return self.notebook.get_current_page() == self.PAGE_APPLIST
+
+    def show_app(self, app):
+        """ Display an application in the installed_pane """
+        self.navigation_bar.add_with_id(_("Installed Software"), self.on_navigation_list, "list", do_callback=False, animate=False)
+        self.navigation_bar.remove_all(do_callback=False, animate=False) # do_callback and animate *must* both be false here
+        details = app.get_details(self.db)
+        self.navigation_bar.add_with_id(details.display_name,
+                                        self.on_navigation_details,
+                                        "details",
+                                        animate=False)
+        self.app_details.show_app(app)
+        self.app_view.emit("application-selected", app)
 
 if __name__ == "__main__":
+
+    from softwarecenter.apt.apthistory import get_apt_history
+    from softwarecenter.db.database import StoreDatabase
+
     #logging.basicConfig(level=logging.DEBUG)
-    xapian_base_path = XAPIAN_BASE_PATH
-    pathname = os.path.join(xapian_base_path, "xapian")
 
     if len(sys.argv) > 1:
         datadir = sys.argv[1]
@@ -184,13 +222,46 @@ if __name__ == "__main__":
     else:
         datadir = "/usr/share/software-center"
 
-    db = xapian.Database(pathname)
+    # additional icons come from app-install-data
     icons = gtk.icon_theme_get_default()
-    icons.append_search_path("/usr/share/app-install/icons/")
-    cache = apt.Cache(apt.progress.OpTextProgress())
+    icons.append_search_path(ICON_PATH)
+    icons.append_search_path(os.path.join(datadir,"icons"))
+    icons.append_search_path(os.path.join(datadir,"emblems"))
+    # HACK: make it more friendly for local installs (for mpt)
+    icons.append_search_path(datadir+"/icons/32x32/status")
+    gtk.window_set_default_icon_name("softwarecenter")
+    cache = apt.Cache(apt.progress.text.OpProgress())
     cache.ready = True
 
-    w = InstalledPane(cache, db, icons, datadir)
+    #apt history
+    history = get_apt_history()
+    # xapian
+    xapian_base_path = XAPIAN_BASE_PATH
+    pathname = os.path.join(xapian_base_path, "xapian")
+    try:
+        db = StoreDatabase(pathname, cache)
+        db.open()
+    except xapian.DatabaseOpeningError:
+        # Couldn't use that folder as a database
+        # This may be because we are in a bzr checkout and that
+        #   folder is empty. If the folder is empty, and we can find the
+        # script that does population, populate a database in it.
+        if os.path.isdir(pathname) and not os.listdir(pathname):
+            from softwarecenter.db.update import rebuild_database
+            logging.info("building local database")
+            rebuild_database(pathname)
+            db = StoreDatabase(pathname, cache)
+            db.open()
+    except xapian.DatabaseCorruptError, e:
+        logging.exception("xapian open failed")
+        view.dialogs.error(None, 
+                           _("Sorry, can not open the software database"),
+                           _("Please re-install the 'software-center' "
+                             "package."))
+        # FIXME: force rebuild by providing a dbus service for this
+        sys.exit(1)
+
+    w = InstalledPane(cache, history, db, 'Ubuntu', icons, datadir)
     w.show()
 
     win = gtk.Window()
